import java.awt.Color;
import java.util.Random;

/**
 * Class that represents a deer.  The deer class
 * tracks all living deer with a linked list.
 * 
 * @author Barb Ericson ericson@cc.gatech.edu
 */
public class Deer extends Turtle
{
  
  /////////////// fields //////////////////////
  
  /** class constant for the color */
  private static final Color brown = new Color(116,64,35);
  
  /** class constant for probability of turning */
  private static final double PROB_OF_TURN = 1/5;
  
  /** class constant for top speed (max num steps can move in a timestep) */
  private static final int maxSpeed = 50;
  
  /** random number generator */
  private static Random randNumGen = new Random();
  
  /** the simulation I'm in */
  private WolfDeerSimulation mySim;
  
  ////////////////////////////// Constructors ////////////////////////
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomally assigned
   * @param modelDisplayer thing which will display the model
   */
  public Deer (ModelDisplay modelDisplayer,WolfDeerSimulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer);
    init(thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   */
  public Deer (int x, int y, ModelDisplay modelDisplayer,
               WolfDeerSimulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer);
    init(thisSim);
  }
  
  ////////////////// methods ////////////////////////////////////////
    
  /**
   * Method to initialize the new deer object
   */
  public void init(WolfDeerSimulation thisSim)
  {
    // set the color of this deer
    setColor(brown);
        
    // turn some random direction
    this.turn(randNumGen.nextInt(360));
    
    // know my simulation
    mySim = thisSim;
  }
  
  /**
   * Method to act during a time step 
   * pick a random direction and move some random amount up to top speed
   */
  public void act()
  {
    // if the random number is > prob of turning then turn
    if (randNumGen.nextFloat() > PROB_OF_TURN)
    {
        this.turn(randNumGen.nextInt(360));
    }
    
    // go forward some random amount 
    forward(randNumGen.nextInt(maxSpeed));
  }
  
  /**
   * Method that handles when a deer dies
   */
  public void die() 
  { 
    // Leave a mark on the world where I died...
    this.setBodyColor(Color.red); 
    /*this.setPenDown(true);
    this.forward(50); this.turn(120); 
    this.forward(50); this.turn(120); 
    this.forward(50); this.turn(120);*/
    
    // Remove me from the "live" list
    mySim.getDeer().remove(this);
    
    // ask the model display to remove this
    // Think of this as "ask the viewable world to remove this turtle"
    //getModelDisplay().remove(this);
    
    System.out.println("<SIGH!> A deer died...");
  }
  
  
}